package img.connection;

import java.io.*;
import java.net.*;
import java.nio.*;
import java.nio.channels.*;
import java.util.*;

import javax.comm.*;

/***********************************************************************
 * This class represents a Connection to a Serial Port.
 */
public class SerialConnection extends AbstractConnection {

   private SerialPort serialPort;
   private SerialParams params;
   private String comPort;

   public static String[] getSerialPorts() {

      ArrayList ports = new ArrayList();

      Enumeration allPorts = CommPortIdentifier.getPortIdentifiers();
      CommPortIdentifier identifier;

      while(allPorts.hasMoreElements()) {
         identifier = (CommPortIdentifier)allPorts.nextElement();
         if(identifier.getPortType() == CommPortIdentifier.PORT_SERIAL) {
            ports.add(identifier.getName());
         }
      }

      Collections.reverse(ports);

      return (String[])ports.toArray(new String[ports.size()]);

   }

   public SerialConnection(String comPort) {
      this(comPort, new SerialParams());
   }

   public SerialConnection(String comPort, SerialParams params) {
      this.comPort = comPort;
      this.params = params;
   }

   public Object getConnectionObject() {
      return serialPort;
   }

   public synchronized void connect() throws Exception {
      CommPortIdentifier identifier = CommPortIdentifier.getPortIdentifier(comPort);
      serialPort = (SerialPort)identifier.open("Imaging App", 2000);
      serialPort.setFlowControlMode(params.getFlowControl());
      serialPort.setSerialPortParams(params.getBaudRate(),
                                     params.getDataBits(),
                                     params.getStopBits(),
                                     params.getParity());
      setConnected();
   }

   public synchronized void disconnect() throws IOException {
      serialPort.close();
      setDisconnected();
   }

   public InputStream getInputStream() throws IOException {
      return serialPort.getInputStream();
   }

   public OutputStream getOutputStream() throws IOException {
      return serialPort.getOutputStream();
   }

   public ByteChannel getChannel() {
      return null;
   }

   public String toString() {
      return serialPort.toString();
   }

   public void setSerialParams(SerialParams params) throws UnsupportedCommOperationException {
      serialPort.setFlowControlMode(params.getFlowControl());
      serialPort.setSerialPortParams(params.getBaudRate(),
                                     params.getDataBits(),
                                     params.getStopBits(),
                                     params.getParity());
      this.params = params;
   }

   public String getComPort() {
      return comPort;
   }

   public SerialParams getSerialParams() {
      return params;
   }

}
