package img.swing;

import javax.swing.SpinnerNumberModel;

public class WrapSpinnerNumberModel extends SpinnerNumberModel {

   public WrapSpinnerNumberModel(Number value, Comparable minimum,
                                 Comparable maximum, Number stepSize) {
      super(value, minimum, maximum, stepSize);
   }

   public WrapSpinnerNumberModel(int value, int minimum,
                                 int maximum, int stepSize) {
      super(new Integer(value), new Integer(minimum),
            new Integer(maximum), new Integer(stepSize));
   }

   public Object getNextValue() {
      return increment(+1);
   }

   public Object getPreviousValue() {
      return increment(-1);
   }

   private Number increment(int dir) {
      Number newValue;
      if ((getValue() instanceof Float) || (getValue() instanceof Double)) {
         double v = ((Number)getValue()).doubleValue() +
            (((Number)getStepSize()).doubleValue() * (double)dir);
         if (getValue() instanceof Double) {
            newValue = new Double(v);
         } else {
            newValue = new Float(v);
         }
      } else {
         long v = ((Number)getValue()).longValue() +
                  (((Number)getStepSize()).longValue() * (long)dir);

         if (getValue() instanceof Long) {
            newValue = new Long(v);
         } else if (getValue() instanceof Integer) {
            newValue = new Integer((int)v);
         } else if (getValue() instanceof Short) {
            newValue = new Short((short)v);
         } else {
            newValue = new Byte((byte)v);
         }
      }

      if ((getMaximum() != null) && (((Comparable)getMaximum()).compareTo(newValue) < 0)) {
         return (Number)getMinimum();
      }

      if ((getMinimum() != null) && (((Comparable)getMinimum()).compareTo(newValue) > 0)) {
         return (Number)getMaximum();
      }

      return newValue;
   }

}

