package log.swing;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.logging.*;
import java.util.zip.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import org.jdom.*;
import org.jdom.input.*;

import log.swing.filechooser.*;
import log.swing.table.*;
import javax.swing.event.ChangeEvent;

public class JLogApplet extends JApplet {

   public static void main(String[] args) throws Exception {

      final JLogApplet log = new JLogApplet();

      // The JFileChooser takes about 2 seconds to fully load.
      // to keep the user from waiting, load the JFileChooser
      // in a background thread while the JLogFrame is loading.
      Thread loadFileChooser = new Thread(new Runnable() {
         public void run() {
            log.createJFileChooser();
         }
      });

      loadFileChooser.setName("Load File Chooser");
      loadFileChooser.start();

      JFrame frame = new JFrame("System Log");


      log.init();

      Dimension dim = log.getToolkit().getScreenSize();
      // set the log size to 3/4 the screen size and
      // center the log window to the middle of the screen.
      frame.setBounds(dim.width / 8, dim.height / 8,
                      dim.width * 3 / 4, dim.height * 3 / 4);

      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.getContentPane().add(log);
      frame.show();

   }

   private LogTableModel model;
   private boolean isStandAlone;
   private JFileChooser fileChooser;

   public String getParameter(String key) {
      return isStandAlone ? System.getProperty(key) : super.getParameter(key);
   }

   public void init() {
      init(true);
   }

   public void init(boolean isStandAlone) {
      this.isStandAlone = isStandAlone;

      JPanel logLevelP = new JPanel();

      JRadioButton severeRB = new JRadioButton(Level.SEVERE.getLocalizedName());
      JRadioButton warningRB = new JRadioButton(Level.WARNING.getLocalizedName());
      JRadioButton infoRB = new JRadioButton(Level.INFO.getLocalizedName());
      JRadioButton configRB = new JRadioButton(Level.CONFIG.getLocalizedName(), true);
      JRadioButton fineRB = new JRadioButton(Level.FINE.getLocalizedName());
      JRadioButton finerRB = new JRadioButton(Level.FINER.getLocalizedName());
      JRadioButton finestRB = new JRadioButton(Level.FINEST.getLocalizedName());

      JCheckBox onlyShowLevelCB = new JCheckBox("Show Only Level", false);
      onlyShowLevelCB.addChangeListener(new OnlyShowLevelAction());

      ButtonGroup bg = new ButtonGroup();
      bg.add(severeRB);
      bg.add(warningRB);
      bg.add(infoRB);
      bg.add(configRB);
      bg.add(fineRB);
      bg.add(finerRB);
      bg.add(finestRB);

      LogAction logAction = new LogAction();

      severeRB.addActionListener(logAction);
      warningRB.addActionListener(logAction);
      infoRB.addActionListener(logAction);
      configRB.addActionListener(logAction);
      fineRB.addActionListener(logAction);
      finerRB.addActionListener(logAction);
      finestRB.addActionListener(logAction);

      logLevelP.add(new JLabel("Level: "));
      logLevelP.add(severeRB);
      logLevelP.add(warningRB);
      logLevelP.add(infoRB);
      logLevelP.add(configRB);
      logLevelP.add(fineRB);
      logLevelP.add(finerRB);
      logLevelP.add(finestRB);
      logLevelP.add(Box.createHorizontalStrut(20));
      logLevelP.add(onlyShowLevelCB);

      getContentPane().add(logLevelP, BorderLayout.NORTH);

      model = new LogTableModel(Level.CONFIG);

      JTable table = new JTable(model);
      table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
      table.setDefaultRenderer(String.class, new LogTableCellRenderer());
      table.setRowHeight(table.getRowHeight() + 5);

      model.addTableModelListener(new LogModelHandler(table));

      TableColumn column = table.getColumnModel().getColumn(0);
      column.setPreferredWidth(200);
      column.setMaxWidth(200);

      column = table.getColumnModel().getColumn(1);
      column.setPreferredWidth(75);
      column.setMaxWidth(75);

      JScrollPane tableScroll = new JScrollPane(table);
      tableScroll.getViewport().setBackground(table.getBackground());

      setJMenuBar(createMenuBar());

      getContentPane().add(tableScroll, BorderLayout.CENTER);

   }

   // overwritten so this applet can be resized using javascript
   public void setSize(int width, int height) {
      Dimension size = getSize();
      if(size.width != width || size.height != height) {
         super.setSize(width, height);
         validate();
      }
   }

   public JFrame createFrame() {
      JFrame frame = new JFrame("System Log");

      frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
      frame.getContentPane().add(this);

      return frame;
   }

   public JDialog createDialog(Frame owner) {

      JDialog dialog = new JDialog(owner,
                                   "System Log",
                                   false);

      dialog.getContentPane().add(this);

      return dialog;

   }

   private JMenuBar createMenuBar() {

      JMenuBar menubar = new JMenuBar();

      JMenu fileM = new JMenu("File");
      fileM.setMnemonic('f');

      try {
         if(isStandAlone) {
            // only allow user to open a log file if the log is running stand-alone
            fileM.add(new ProxyAction(this, "open",
                                      "Open",
                                      'o'));
         } else {
            // only allow the user to clear the log if the log is NOT running stand-alone
            fileM.add(new ProxyAction(this, "clear",
                                      "Clear",
                                      'c'));
         }
      } catch(NoSuchMethodException nsme) {
      }

      menubar.add(fileM);

      return menubar;
   }

   public LogTableModel getTableModel() {
      return model;
   }

   public void open() {

      if(fileChooser == null) {
         createJFileChooser();
      }

      int option = fileChooser.showOpenDialog(getRootPane());
      if(option == JFileChooser.APPROVE_OPTION) {

         clear();

         try {

            getRootPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            File logFile = fileChooser.getSelectedFile();

            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(new GZIPInputStream(new FileInputStream(logFile)));

            Element root = doc.getRootElement();

            Iterator records = root.getChildren("record").iterator();
            Element recordElement;
            LogRecord record;

            while(records.hasNext()) {

               recordElement = (Element)records.next();

               record = new LogRecord(Level.parse(recordElement.getChildText("level")), recordElement.getChildText("message"));
               record.setMillis(Long.parseLong(recordElement.getChildText("millis")));
               record.setSequenceNumber(Long.parseLong(recordElement.getChildText("sequence")));
               record.setLoggerName(recordElement.getChildText("logger"));
               record.setSourceClassName(recordElement.getChildText("class"));
               record.setSourceMethodName(recordElement.getChildText("method"));
               record.setThreadID(Integer.parseInt(recordElement.getChildText("thread")));

               model.addRecord(record);

            }


         } catch(Exception e) {
            e.printStackTrace(System.err);
         }

         getRootPane().setCursor(Cursor.getDefaultCursor());

      }

   }

   public void save() {

      if(model.getRowCount() == 0) {
         return;
      }

      if(fileChooser == null) {
         createJFileChooser();
      }

      int option = fileChooser.showSaveDialog(getRootPane());
      if(option == JFileChooser.APPROVE_OPTION) {
         try {
            File logFile = fileChooser.getSelectedFile();

            if(!logFile.getName().endsWith(".log")) {
               logFile = new File(logFile.getAbsolutePath() + ".log");
            }

            FileOutputStream outStream = new FileOutputStream(logFile);
            ObjectOutputStream out = new ObjectOutputStream(outStream);

            int rows = model.getRowCount();
            out.writeInt(rows);

            for(int i = 0; i < rows; i++) {
               out.writeObject(model.getValueAt(i, 0));
            }

            out.close();

         } catch(Exception e) {
            e.printStackTrace(System.err);
         }
      }

   }

   private void createJFileChooser() {

      getRootPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      System.out.println("Loading File Chooser... " + Thread.currentThread().getName());

      if(fileChooser != null) {
         return;
      }

      File logDir = new File("/var/log");
      if(!logDir.exists()) {
         logDir.mkdirs();
      }
      fileChooser = new JFileChooser(logDir);
      fileChooser.removeChoosableFileFilter(fileChooser.getFileFilter());
      fileChooser.setFileFilter(new LogFileFilter("Log File (*.log)", ".log"));

      System.out.println("File Chooser Loaded... " + Thread.currentThread().getName());
      getRootPane().setCursor(Cursor.getDefaultCursor());

   }

   public void clear() {
      model.clear();
   }

   private class LogAction implements ActionListener {

      public void actionPerformed(ActionEvent ae) {
         model.setFilter(Level.parse(ae.getActionCommand()));
      }

   }

   private class OnlyShowLevelAction implements ChangeListener {

      public void stateChanged(ChangeEvent changeEvent) {
         model.setOnlyShowSelectedLevel(((JCheckBox)changeEvent.getSource()).isSelected());
      }

   }

   private static class LogModelHandler implements TableModelListener {

      private JTable table;

      public LogModelHandler(JTable table) {
         this.table = table;
      }

      public void tableChanged(TableModelEvent tableModelEvent) {
         table.scrollRectToVisible(table.getCellRect(table.getRowCount() - 1, 0, true));
      }

   }

}
